use crate::utils::{get_dna_complement, read_fasta};

fn get_revps(dna: &str, complement: &str) -> Vec<usize> {
    let mut out_vec: Vec<usize> = Vec::new();
    for i in 4..dna.len() + 1 {
        let sub_rev_comp: &String = &complement[..i].chars().rev().collect();
        if sub_rev_comp == &dna[..i] {
            out_vec.push(sub_rev_comp.len())
        }
    }
    out_vec
}

pub fn revp(file_content: String) -> String {
    let fastas = read_fasta(&file_content);
    let fasta = &fastas[0];
    let complement = get_dna_complement(&fasta.sequence);
    let mut out_vec: Vec<String> = Vec::new();
    let dna_len = fasta.sequence.len();

    for position in 1..dna_len - 2 {
        let mut last_pos = position + 11;
        if last_pos > dna_len {
            last_pos = dna_len;
        }
        let revps = get_revps(
            &fasta.sequence[position - 1..last_pos],
            &complement[position - 1..last_pos],
        );
        for r in revps.iter() {
            out_vec.push(format!("{} {}", position, r));
        }
    }

    return out_vec.join("\n");
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_revp() {
        let input_content = r#">Rosalind_24
TCAATGCATGCGGGTCTATATGCAT"#
            .to_string();
        let expected = r#"4 6
5 4
6 6
7 4
17 4
18 4
20 6
21 4"#
            .to_string();
        assert_eq!(revp(input_content), expected);
    }
}
